#include <stdio.h>
#include <errno.h>
#include <assert.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/ipc.h>
#include <sys/sem.h>

/* We must define union semun ourselves.  */
union semun {
  int val;
  struct semid_ds *buf;
  unsigned short int *array;
  struct seminfo *__buf;
};

/* Wait on a semaphore. Block until the semaphore value is positive, then decrement.  */
void wait (int semid)
{
  struct sembuf operations[1];
  operations[0].sem_num = 0;         /* Use the first (and only) semaphore.  */
  operations[0].sem_op = -1;         /* Decrement by 1.  */
  operations[0].sem_flg = SEM_UNDO;  /* Permit undo'ing.  */  

  if (semop (semid, operations, 1))
    fprintf(stderr, "%s\n", strerror(errno));

   
}

/* Post to a binary semaphore: increment its value. This returns immediately. */
void post (int semid)
{
  struct sembuf operations[1];  
  operations[0].sem_num = 0;        /* Use the first (and only) semaphore.  */ 
  operations[0].sem_op = 1;         /* Increment by 1.  */
  operations[0].sem_flg = SEM_UNDO; /* Permit undo'ing.  */  


  if (semop (semid, operations, 1))
    fprintf(stderr, "%s\n", strerror(errno));
}

int main()
{
  int rc;
  union semun argument;

  /* Obtain a binary semaphore's ID, allocating if necessary.  */
  int sem_id = semget(IPC_PRIVATE, 1, O_CREAT | S_IRWXU );
  printf("Semaphore id = %i\n", sem_id);

  /* Initialize the semaphore with a value of one.  */  
  unsigned short values[1];
  values[0] = 1;
  argument.array = values;
  if (semctl(sem_id, 0, SETALL, argument))
    fprintf(stderr, "%s\n", strerror(errno));

  if (!fork())
    { /* now in child process */
      while (1)
	{
	  wait(sem_id);	  
	  putchar(8);
	  putchar(' ');
	  putchar(8);
	  fflush(stdout);
	  sleep(rand()%3);
	  fflush(stdout);
	}
  
    } else { 
      /* in parent process */
      while (1)
	{
	  printf("+");
	  fflush(stdout);
	  post(sem_id);	  
	  sleep(1);
	}
    }

  /* Deallocate a binary semaphore. All users must have finished their use. 
     The second argument is ignored. Returns -1 on failure.  */
  if (semctl (sem_id, 1, IPC_RMID)==-1)
    fprintf(stderr, "%s\n", strerror(errno));

}
