/***********************************************************************
* Code listing from "Advanced Linux Programming," by CodeSourcery LLC  *
* Copyright (C) 2001 by New Riders Publishing                          *
* See COPYRIGHT for license information.                               *
***********************************************************************/

#include <assert.h>
#include <stdio.h>
#include <sys/time.h>
#include <time.h>
#include <string.h>

#include "server.h"

#define THIS_MODULE "time"

/* A template for the HTML page this module generates.  */

static char* page_template =
  "<html>\n"
  " <head>\n"
  "  <meta http-equiv=\"refresh\" content=\"10\">\n"
  " </head>"
  " <body>"
  "  <p>Time and date will be formatted according to the following format: <pre> %s</pre></p>"
  "  <p>The current date and time, formatted as requested, is: <br><h1>%s</h1></p>\n"
  " <p>"
  "<form action=\"%s\" method=\"GET\">"
  " Select the desired format: <select name=\"format\">"
  "  <option value=\"%%R\"> 23:59"
  "  <option value=\"%%T\"> 23:59:59"
  "  <option value=\"%%r\"> 11:59:59 p"
  "  <option value=\"%%Y-%%m-%%d\"> 2000-12-31"
  "  <option value=\"%%a,  %%d  %%b  %%Y  %%H:%%M:%%S  %%z\"> Sat, 31 Dec 2000 23:59:59 +0100"
  "  <option value=\"%%a,  %%d  %%b  %%Y\"> Sat, 31 Dec 2000"
  " </select>"
  " <input type=\"submit\" value=\"Ok\">"
  "</form>"
  "</body>\n"
  "</html>\n";



char * get_parameter(const char * parameters, char * parameter_name)
{
  char * occurrence, * value;
  int len = strlen(parameter_name);
  if (!parameters || !len) return NULL;

  while (occurrence = strstr(parameters, parameter_name)) {
    /* Check that character before occurrence is a delimiter */
    if (occurrence == parameters || *(occurrence-1)=='&') {
      /* Check that character after occurrence is a '=' */
      if (occurrence[len]=='=') {
	char * end;
	if (end = strchr(occurrence, '&')) {
	  assert(end>occurrence+len);
	  value = (char*) xmalloc(end - (occurrence+len)+1);
	  strncpy(value, occurrence+len+1, end-1-occurrence-len);	    
	  /* Terminating the string */
	  value[end-1-occurrence-len]=0;
	  return value;
	} else return strdup(occurrence + len +1);	     	  
      }
    }
    parameters = occurrence + len + 1;
  }
  return NULL;
}

void module_generate (char * parameters, int fd)
{
  struct timeval tv;
  struct tm* ptm;
  char time_string[40];
  FILE* fp;
  char * format = get_parameter(parameters, "format");
  char * default_format = "%H:%M:%S";
  if (!format) format = default_format;
 
  /* Obtain the time of day, and convert it to a tm struct.  */
  gettimeofday (&tv, NULL);
  ptm = localtime (&tv.tv_sec);

  strftime (time_string, sizeof (time_string), format, ptm);

  /* Create a stream corresponding to the client socket file
     descriptor.  */
  fp = fdopen (fd, "w");
  assert (fp != NULL);
  /* Generate the HTML output.  */
  fprintf (fp, page_template, format, time_string, THIS_MODULE);


  /* All done; flush the stream.  */
  fflush (fp);
}
